/*          ****************
            *FILE:COMPLEX.H*
            ****************        */

/*------------------------------------------------------------------
Action: Header file for the "Complex" method optimization algorithm.

Author: J.A.Wright
Date:5-8-00
--------------------------------------------------------------------*/

/*Search Control Parameters */

#define ALPHA           1.3
#define LOWEST_ALPHA    0.000001
#define THRESH          1E-6
#define MAX_SEARCH_VAR  20
#define INFEAS          1
#define FEAS            0
#define CONTINUE        1
#define STOP            0
#define MONIT           1
#define DONT_MONIT      0
#define CONVERGE        1

/*Algorithm Global Variables*/

    struct complex_method {
        /*The following must be specified by the calling program,
          initial guess of solution, variable bounds, number of variables,
          tollerance on the standard deviation between objective function
          values (used as a measure of the accuracy required),
          the maximum number of function evaluations allowed,
          and a flag monitoring search progress (MONIT or DONT_MONIT).*/
        double x_guess[MAX_SEARCH_VAR],x_lb[MAX_SEARCH_VAR],x_ub[MAX_SEARCH_VAR];
        double deviation_tollerance;
        int    n_var, max_func_call, monit;
        /*Internally used variables*/
        double vertex[(2*MAX_SEARCH_VAR+2)][MAX_SEARCH_VAR];
        double worst[MAX_SEARCH_VAR],centroid[MAX_SEARCH_VAR];
        double f[(2*MAX_SEARCH_VAR+2)];
        int    n_func_calls;
        /*Optimum solution*/
        double optimum[(2*MAX_SEARCH_VAR+2)],f_optimum;
    };


/*Complex Method*/
void box(void);
void initialize_box(void);
void calc_centroid(int i_worst,int n_vert);
void test_and_move(int i_vertex);
void obj_sort(double objective[],int n_obj,int *sort);
void copy_worst(int i_worst);
void reflect(int i_worst, double alpha);
void reset_worst(int i_worst);
int  converge(void);
void copy_optimum(int i_best);


/*Complex Method Objective and Constraint Functions*/
double calc_objective(double x[]);
int    test_constraints(double x[]);

/*Random Number Functions*/
double jrandom(void);
